@@ -3,9 +3,9 @@ ConvertType - Convert between various types of files.
 ConvertType is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016, Free Software Foundation, Inc.
+Copyright (C) 2016-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -28,10 +28,14 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <stdio.h>
 #include <string.h>
 
+#include <gnuastro/eps.h>
 #include <gnuastro/txt.h>
 #include <gnuastro/wcs.h>
+#include <gnuastro/pdf.h>
 #include <gnuastro/list.h>
 #include <gnuastro/fits.h>
+#include <gnuastro/jpeg.h>
+#include <gnuastro/tiff.h>
 #include <gnuastro/table.h>
 #include <gnuastro/blank.h>
 #include <gnuastro/arithmetic.h>
@@ -44,8 +48,6 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include "main.h"
 
 #include "ui.h"
-#include "eps.h"
-#include "jpeg.h"
 #include "authors-cite.h"
 
 
@@ -80,16 +82,6 @@ doc[] = GAL_STRINGS_TOP_HELP_INFO PROGRAM_NAME" will convert any of the "
 
 
 
-/* Option groups particular to this program */
-enum program_args_groups
-{
-  ARGS_GROUP_FLUX = GAL_OPTIONS_GROUP_AFTER_COMMON,
-};
-
-
-
-
-
 
 
 
@@ -234,6 +226,85 @@ parse_opt(int key, char *arg, struct argp_state *state)
 /**************************************************************/
 /***************       Sanity Check         *******************/
 /**************************************************************/
+static void
+ui_colormap_sanity_check(struct converttparams *p)
+{
+  char **strarr;
+  float *farray;
+  size_t nparams=0;
+  int ccode=COLOR_INVALID;
+
+  /* See how many parameters are necessary. */
+  strarr=p->colormap->array;
+  if     ( !strcmp(strarr[0],"hsv" )) { ccode=COLOR_HSV;  nparams=2; }
+  else if( !strcmp(strarr[0],"sls" )) { ccode=COLOR_SLS;  nparams=0; }
+  else if( !strcmp(strarr[0],"gray") || !strcmp(strarr[0],"grey"))
+                                      { ccode=COLOR_GRAY; nparams=0; }
+  else
+    error(EXIT_FAILURE, 0, "`%s' not recognized as a color-space given "
+          "to `--monotocolor'", strarr[0]);
+  p->colormap->status=ccode;
+
+  /* Check if the proper number of parameters are given for this color
+     space. Note that the actual colorspace name is the first element in
+     `monotocolor'. */
+  if(p->colormap->size!=1 && p->colormap->size != (nparams+1) )
+    error(EXIT_FAILURE, 0, "%zu parameters given to `--monotocolor' for "
+          "the `%s' color space (which needs %zu)",
+          p->colormap->size-1, strarr[0], nparams);
+
+  /* Allocate the necessary space for the parameters (when
+     necessary). */
+  if(nparams>0)
+    {
+      /* If no parameters were given, put the full range. */
+      if(p->colormap->size==1)
+        {
+          p->colormap->next=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1,
+                                            &nparams, NULL, 0,
+                                            p->cp.minmapsize, NULL,NULL,NULL);
+          farray=p->colormap->next->array;
+          switch(p->colormap->status)
+            {
+            case COLOR_HSV: farray[0]=0; farray[1]=360; break;
+            default:
+              error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at "
+                    "%s to fix the problem. The value `%d' is not "
+                    "recognized for a color space that needs default "
+                    "parameters", __func__, PACKAGE_BUGREPORT,
+                    p->colormap->status);
+            }
+        }
+      else
+        {
+          /* Increment the array pointer (temporarily) so we can read
+             the rest of the parameters as float32. Note that we can't
+             use `+=' because it is a `void *' pointer. We'll have to
+             use `strarry', because its type is defined. */
+          p->colormap->size-=1;
+          p->colormap->array = strarr + 1;
+          p->colormap->next=gal_data_copy_to_new_type(p->colormap,
+                                                       GAL_TYPE_FLOAT32);
+          p->colormap->array = strarr;
+          p->colormap->size+=1;
+
+          /* For a check
+             {
+               size_t i;
+               farray=p->monotocolor->next->array;
+               for(i=0;i<p->monotocolor->next->size;++i)
+               printf("%f\n", farray[i]);
+               exit(1);
+             }
+          */
+        }
+    }
+}
+
+
+
+
+
 /* Read and check ONLY the options. When arguments are involved, do the
    check in `ui_check_options_and_arguments'. */
 static void
@@ -270,6 +341,10 @@ ui_read_check_only_options(struct converttparams *p)
 
       gal_data_free(cond);
     }
+
+  /* Check the colormap. */
+  if(p->colormap)
+    ui_colormap_sanity_check(p);
 }
 
 
@@ -279,14 +354,11 @@ ui_read_check_only_options(struct converttparams *p)
 static void
 ui_check_options_and_arguments(struct converttparams *p)
 {
-  /* Check if there was any inputs. */
-  if(p->inputnames==NULL)
-    error(EXIT_FAILURE, 0, "no input files given");
-
-  /* Reverse the `inputnames' linked list, note that the `hdu' linked list
-     was reversed during option parsing.*/
+  /* Reverse the `inputnames' linked list if it was given (recall that we
+     also accept input from the standard input). Note that the `hdu' linked
+     list was reversed during option parsing, so we don't need to do it
+     here any more. */
   gal_list_str_reverse(&p->inputnames);
-
 }
 
 
@@ -403,12 +475,28 @@ static void
 ui_make_channels_ll(struct converttparams *p)
 {
   char *hdu=NULL;
-  size_t dsize=0;
   gal_data_t *data;
-  gal_list_str_t *name;
+  size_t dsize=0, dirnum;
+  gal_list_str_t *name, *lines;
+
+  /* Initialize the counting of channels. */
+  p->numch=0;
+
+  /* If any standard input is provided, we want to process that first. Note
+     that since other input arguments are also allowed (as other channels),
+     we'll need to process the standard input independently first, then go
+     onto the possible list of other files.*/
+  lines=gal_txt_stdin_read(p->cp.stdintimeout);
+  if(lines)
+    {
+      data=gal_txt_image_read(NULL, lines, p->cp.minmapsize);
+      gal_list_data_add(&p->chll, data);
+      gal_list_str_free(lines, 1);
+      ++p->numch;
+    }
+
 
   /* Go through the input files and add the channel(s). */
-  p->numch=0;
   for(name=p->inputnames; name!=NULL; name=name->next)
     {
       /* Check if p->numch has not exceeded 4. */
@@ -442,19 +530,31 @@ ui_make_channels_ll(struct converttparams *p)
         }
 
 
+      /* TIFF: */
+      else if( gal_tiff_name_is_tiff(name->v) )
+        {
+          /* Get the directory value for this channel. */
+          if(p->hdus)
+            {
+              hdu=gal_list_str_pop(&p->hdus);
+              dirnum=gal_tiff_dir_string_read(hdu);
+            }
+          else
+            dirnum=0;
+
+          /* Read the TIFF image into memory. */
+          data=gal_tiff_read(name->v, dirnum, p->cp.minmapsize);
+          p->numch += gal_list_data_number(data);
+          gal_list_data_add(&p->chll, data);
+        }
+
 
       /* JPEG: */
-      else if ( nameisjpeg(name->v) )
+      else if ( gal_jpeg_name_is_jpeg(name->v) )
         {
-#ifndef HAVE_LIBJPEG
-          error(EXIT_FAILURE, 0, "you are giving a JPEG input, however, "
-                "when %s was configured libjpeg was not available. To read "
-                "from (and write to) JPEG files, libjpeg is required. "
-                "Please install it and configure, make and install %s "
-                "again", PACKAGE_STRING, PACKAGE_STRING);
-#else
-          p->numch += jpeg_read_to_ll(name->v, &p->chll, p->cp.minmapsize);
-#endif
+          data=gal_jpeg_read(name->v, p->cp.minmapsize);
+          p->numch += gal_list_data_number(data);
+          gal_list_data_add(&p->chll, data);
         }
 
 
@@ -471,29 +571,33 @@ ui_make_channels_ll(struct converttparams *p)
 
 
       /* EPS:  */
-      else if ( nameiseps(name->v) )
+      else if ( gal_eps_name_is_eps(name->v) )
         error(EXIT_FAILURE, 0, "EPS files cannot be used as input. Since "
-              "EPS files are not raster graphics, they are only used as "
-              "output");
+              "EPS files are not raster graphics. EPS is only an output "
+              "format");
 
 
 
       /* PDF:  */
-      else if ( nameispdf(name->v) )
+      else if ( gal_pdf_name_is_pdf(name->v) )
         error(EXIT_FAILURE, 0, "PDF files cannot be used as input. Since "
-              "PDF files are not raster graphics, they are only used as "
-              "output");
+              "PDF files are not raster graphics. PDF is only an output "
+              "format");
 
 
       /* Text: */
       else
         {
-          data=gal_txt_image_read(name->v, p->cp.minmapsize);
+          data=gal_txt_image_read(name->v, NULL, p->cp.minmapsize);
           gal_list_data_add(&p->chll, data);
           ++p->numch;
         }
     }
 
+  /* If there weren't any channels, abort with an error. */
+  if(p->numch==0)
+    error(EXIT_FAILURE, 0, "%s",
+          gal_options_stdin_error(p->cp.stdintimeout, 0, "input"));
 
   /* Reverse the list of channels into the input order. */
   gal_list_data_reverse(&p->chll);
@@ -525,6 +629,27 @@ ui_prepare_input_channels(struct converttparams *p)
           "contain more than one color channel", p->numch);
 
 
+  /* If there are multiple color channels, then ignore the monotocolor
+     option if it is given. But if there is only one, make sure that the
+     `colormap' option is actually given.*/
+  if( p->numch==1 )
+    {
+      if( p->colormap==NULL )
+        error(EXIT_FAILURE, 0, "no colormap! When there is only one input "
+              "channel, it is necessary to specify a color map. For "
+              "example `gray', `hsv', or `sls'.\n\n"
+              "For more on ConvertType's color mapping, see the "
+              "description under `--colormap' in the Gnuastro book:\n\n"
+              "   $ info astconvertt");
+    }
+  else if( p->numch>1 && p->colormap )
+    {
+      if(p->colormap->next) gal_data_free(p->colormap->next);
+      gal_data_free(p->colormap);
+      p->colormap=NULL;
+    }
+
+
   /* Go over the channels and make the proper checks/corrections. We won't
      be checking blank channels here, recall that blank channels had a
      dimension of zero. */
@@ -612,7 +737,7 @@ void
 ui_add_dot_use_automatic_output(struct converttparams *p)
 {
   gal_list_str_t *stll;
-  char *tmp, *firstname="output.txt", *suffix=p->cp.output;
+  char *tmp, *firstname="converttype.txt", *suffix=p->cp.output;
 
   /* Find the first non-blank file name in the input(s). */
   for(stll=p->inputnames; stll!=NULL; stll=stll->next)
@@ -625,7 +750,8 @@ ui_add_dot_use_automatic_output(struct converttparams *p)
   /* If the suffix does not start with a `.', put one there. */
   if(suffix[0]!='.')
     {
-      asprintf(&tmp, ".%s", suffix);
+      if( asprintf(&tmp, ".%s", suffix)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       free(suffix);
       suffix=tmp;
     }
@@ -646,49 +772,61 @@ ui_set_output(struct converttparams *p)
 {
   struct gal_options_common_params *cp=&p->cp;
 
-  /* Determine the type */
+  /* FITS */
   if(gal_fits_name_is_fits(cp->output))
     {
       p->outformat=OUT_FORMAT_FITS;
       if( gal_fits_suffix_is_fits(cp->output) )
         ui_add_dot_use_automatic_output(p);
     }
-  else if(nameisjpeg(cp->output))
+
+  /* JPEG */
+  else if(gal_jpeg_name_is_jpeg(cp->output))
     {
-#ifndef HAVE_LIBJPEG
-      error(EXIT_FAILURE, 0, "you have asked for a JPEG output, "
-            "however, when %s was configured libjpeg was not "
-            "available. To write to JPEG files, libjpeg is required. "
-            "Please install it and configure, make and install %s "
-            "again", PACKAGE_STRING, PACKAGE_STRING);
-#else
       /* Small sanity checks. */
       if(p->quality == GAL_BLANK_UINT8)
-        error(EXIT_FAILURE, 0, "the `--quality' (`-u') option is necessary for "
-              "jpeg outputs, but it has not been given");
+        error(EXIT_FAILURE, 0, "the `--quality' (`-u') option is necessary "
+              "for jpeg outputs, but it has not been given");
       if(p->quality > 100)
         error(EXIT_FAILURE, 0, "`%u' is larger than 100. The value to the "
-              "`--quality' (`-u') option must be between 1 and 100 (inclusive)",
-              p->quality);
+              "`--quality' (`-u') option must be between 1 and 100 "
+              "(inclusive)", p->quality);
 
       /* Preparations. */
       p->outformat=OUT_FORMAT_JPEG;
-      if( nameisjpegsuffix(cp->output) )
+      if( gal_jpeg_suffix_is_jpeg(cp->output) )
         ui_add_dot_use_automatic_output(p);
-#endif
     }
-  else if(nameiseps(cp->output))
+
+  /* TIFF */
+  else if( gal_tiff_name_is_tiff(cp->output) )
+      error(EXIT_FAILURE, 0, "writing TIFF files is not yet supported, "
+            "please get in touch with us at %s so we implement it",
+            PACKAGE_BUGREPORT);
+
+  /* EPS */
+  else if(gal_eps_name_is_eps(cp->output))
     {
+      if(p->borderwidth==0 && p->widthincm==0)
+        error(EXIT_FAILURE, 0, "at least one of `--widthincm' (`-u'), or "
+              "`--borderwidth (`-b') options are necessary for an EPS output");
       p->outformat=OUT_FORMAT_EPS;
-      if( nameisepssuffix(cp->output) )
+      if( gal_eps_suffix_is_eps(cp->output) )
         ui_add_dot_use_automatic_output(p);
     }
-  else if(nameispdf(cp->output))
+
+  /* PDF */
+  else if(gal_pdf_name_is_pdf(cp->output))
     {
+      if(p->borderwidth==0 && p->widthincm==0)
+        error(EXIT_FAILURE, 0, "at least one of `--widthincm' (`-u'), or "
+              "`--borderwidth (`-b') options are necessary for a PDF output");
       p->outformat=OUT_FORMAT_PDF;
-      if( nameispdfsuffix(cp->output) )
+      if( gal_pdf_suffix_is_pdf(cp->output) )
         ui_add_dot_use_automatic_output(p);
     }
+
+  /* Default: plain text. */
   else
     {
       p->outformat=OUT_FORMAT_TXT;
@@ -725,8 +863,8 @@ ui_set_output(struct converttparams *p)
         }
     }
 
-  /* Check if the output already exists. */
-  gal_checkset_check_remove_file(cp->output, 0, cp->dontdelete);
+  /* Check if the output already exists and remove it if allowed. */
+  gal_checkset_writable_remove(cp->output, 0, cp->dontdelete);
 }
 
 
@@ -846,7 +984,11 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct converttparams *p)
 void
 ui_free_report(struct converttparams *p)
 {
-  /* Free the allocated spaces */
+  if(p->colormap)
+    {
+      if(p->colormap->next) gal_data_free(p->colormap->next);
+      gal_data_free(p->colormap);
+    }
   gal_data_free(p->fluxlow);
   gal_data_free(p->fluxhigh);
   gal_list_str_free(p->hdus, 1);
