@@ -3,9 +3,9 @@ MakeCatalog - Make a catalog from an input and labeled image.
 MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016, Free Software Foundation, Inc.
+Copyright (C) 2016-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -32,38 +32,12 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 struct argp_option program_options[] =
   {
     /* Input options. */
-    {
-      "objectsfile",
-      UI_KEY_OBJECTSFILE,
-      "STR",
-      0,
-      "Image containing object/detection labels.",
-      GAL_OPTIONS_GROUP_INPUT,
-      &p->objectsfile,
-      GAL_TYPE_STRING,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
-    {
-      "objectshdu",
-      UI_KEY_OBJECTSHDU,
-      "STR",
-      0,
-      "Object image extension name or number.",
-      GAL_OPTIONS_GROUP_INPUT,
-      &p->objectshdu,
-      GAL_TYPE_STRING,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
     {
       "clumpsfile",
       UI_KEY_CLUMPSFILE,
       "STR",
       0,
-      "Image containing clump labels.",
+      "Dataset containing clump labels.",
       GAL_OPTIONS_GROUP_INPUT,
       &p->clumpsfile,
       GAL_TYPE_STRING,
@@ -76,7 +50,7 @@ struct argp_option program_options[] =
       UI_KEY_CLUMPSHDU,
       "STR",
       0,
-      "Clump image extension name or number.",
+      "Clump labels extension name or number.",
       GAL_OPTIONS_GROUP_INPUT,
       &p->clumpshdu,
       GAL_TYPE_STRING,
@@ -85,11 +59,37 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET
     },
     {
-      "skyfile",
-      UI_KEY_SKYFILE,
+      "valuesfile",
+      UI_KEY_VALUESFILE,
       "STR",
       0,
-      "Image containing sky values.",
+      "Values/brightness dataset.",
+      GAL_OPTIONS_GROUP_INPUT,
+      &p->valuesfile,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "valueshdu",
+      UI_KEY_VALUESHDU,
+      "STR",
+      0,
+      "Name or number of extension containing values.",
+      GAL_OPTIONS_GROUP_INPUT,
+      &p->valueshdu,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "insky",
+      UI_KEY_INSKY,
+      "STR/FLT",
+      0,
+      "Input Sky value or dataset.",
       GAL_OPTIONS_GROUP_INPUT,
       &p->skyfile,
       GAL_TYPE_STRING,
@@ -107,15 +107,28 @@ struct argp_option program_options[] =
       &p->skyhdu,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
     {
-      "stdfile",
-      UI_KEY_STDFILE,
-      "STR",
+      "subtractsky",
+      UI_KEY_SUBTRACTSKY,
       0,
-      "Image containing sky STD values.",
+      0,
+      "Subtract the Sky dataset from the values.",
+      GAL_OPTIONS_GROUP_INPUT,
+      &p->subtractsky,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "instd",
+      UI_KEY_INSTD,
+      "STR/FLT",
+      0,
+      "Sky standard deviation value or dataset.",
       GAL_OPTIONS_GROUP_INPUT,
       &p->stdfile,
       GAL_TYPE_STRING,
@@ -128,12 +141,25 @@ struct argp_option program_options[] =
       UI_KEY_STDHDU,
       "STR",
       0,
-      "Sky image extension name or number.",
+      "Sky STD extension name or number.",
       GAL_OPTIONS_GROUP_INPUT,
       &p->stdhdu,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "variance",
+      UI_KEY_VARIANCE,
+      0,
+      0,
+      "STD input dataset is actually variance.",
+      GAL_OPTIONS_GROUP_INPUT,
+      &p->variance,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
     {
@@ -149,36 +175,36 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+
+
+
+    /* Output. */
     {
-      "skysubtracted",
-      UI_KEY_SKYSUBTRACTED,
+      "clumpscat",
+      UI_KEY_CLUMPSCAT,
       0,
       0,
-      "Input is already sky subtracted (for S/N).",
-      GAL_OPTIONS_GROUP_INPUT,
-      &p->skysubtracted,
+      "Make a clumps catalog also.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->clumpscat,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
     {
-      "threshold",
-      UI_KEY_THRESHOLD,
-      "FLT",
+      "noclumpsort",
+      UI_KEY_NOCLUMPSORT,
       0,
-      "Use pixels more than this multiple of STD.",
-      GAL_OPTIONS_GROUP_INPUT,
-      &p->threshold,
-      GAL_TYPE_FLOAT32,
-      GAL_OPTIONS_RANGE_ANY,
+      0,
+      "Don't sort the clumps catalog by ID.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->noclumpsort,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
-
-
-
-    /* Output. */
     {
       "sfmagnsigma",
       UI_KEY_SFMAGNSIGMA,
@@ -211,8 +237,8 @@ struct argp_option program_options[] =
     /* Upper limit magnitude configurations. */
     {
       0, 0, 0, 0,
-      "Upper limit magnitude:",
-      ARGS_GROUP_UPPERLIMIT
+      "Upper limit magnitude settings:",
+      UI_GROUP_UPPERLIMIT
     },
     {
       "upmaskfile",
@@ -220,7 +246,7 @@ struct argp_option program_options[] =
       "STR",
       0,
       "Mask image file name only for upper limit.",
-      ARGS_GROUP_UPPERLIMIT,
+      UI_GROUP_UPPERLIMIT,
       &p->upmaskfile,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -233,7 +259,7 @@ struct argp_option program_options[] =
       "STR",
       0,
       "Mask image HDU only for upper limit.",
-      ARGS_GROUP_UPPERLIMIT,
+      UI_GROUP_UPPERLIMIT,
       &p->upmaskhdu,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -246,20 +272,34 @@ struct argp_option program_options[] =
       "INT",
       0,
       "Number of randomly positioned samples",
-      ARGS_GROUP_UPPERLIMIT,
+      UI_GROUP_UPPERLIMIT,
       &p->upnum,
       GAL_TYPE_SIZE_T,
       GAL_OPTIONS_RANGE_GT_0,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "uprange",
+      UI_KEY_UPRANGE,
+      "INT,INT",
+      0,
+      "Range of random positions (pix) around target.",
+      UI_GROUP_UPPERLIMIT,
+      &p->uprange,
+      GAL_TYPE_SIZE_T,
+      GAL_OPTIONS_RANGE_GT_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      gal_options_parse_sizes_reverse
+    },
     {
       "envseed",
       UI_KEY_ENVSEED,
       0,
       0,
       "Use GSL_RNG_SEED environment variable for seed.",
-      ARGS_GROUP_UPPERLIMIT,
+      UI_GROUP_UPPERLIMIT,
       &p->envseed,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -272,7 +312,7 @@ struct argp_option program_options[] =
       "FLT,FLT",
       0,
       "Sigma multiple and, tolerance or number.",
-      ARGS_GROUP_UPPERLIMIT,
+      UI_GROUP_UPPERLIMIT,
       &p->upsigmaclip,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -286,23 +326,37 @@ struct argp_option program_options[] =
       "FLT",
       0,
       "Multiple of sigma to define upperlimit.",
-      ARGS_GROUP_UPPERLIMIT,
+      UI_GROUP_UPPERLIMIT,
       &p->upnsigma,
       GAL_TYPE_FLOAT32,
       GAL_OPTIONS_RANGE_GT_0,
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "checkuplim",
+      UI_KEY_CHECKUPLIM,
+      "INT[,INT]",
+      0,
+      "Check random distribution for one label.",
+      UI_GROUP_UPPERLIMIT,
+      &p->checkuplim,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_GT_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_check_upperlimit
+    },
 
 
 
-    /* Output columns: IMPORTANT: Throughout MakeCatalogs, the order of
-       columns is based on this. So if you make changes/additions, please
-       update all the other places the columns are searched. */
+
+
+    /* ID related columns. */
     {
       0, 0, 0, 0,
-      "Output catalog columns",
-      ARGS_GROUP_COLUMNS
+      "Identifier columns",
+      UI_GROUP_COLUMNS_IDS
     },
     {  /* `ids' is not a unique column, it is a combination of several
           columns. */
@@ -311,7 +365,7 @@ struct argp_option program_options[] =
       0,
       0,
       "All IDs of objects and clumps.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_IDS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -325,7 +379,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Object label/ID.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_IDS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -339,7 +393,7 @@ struct argp_option program_options[] =
       0,
       0,
       "ID of object hosting this clump.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_IDS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -353,7 +407,7 @@ struct argp_option program_options[] =
       0,
       0,
       "ID of clump in host object.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_IDS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -361,61 +415,16 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       ui_column_codes_ll
     },
+
+
+
+
+
+    /* Position related columns (pixel). */
     {
-      "numclumps",
-      UI_KEY_NUMCLUMPS,
-      0,
-      0,
-      "Number of clumps in this object.",
-      ARGS_GROUP_COLUMNS,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
-    },
-    {
-      "area",
-      UI_KEY_AREA,
-      0,
-      0,
-      "Number of pixels in clump or object.",
-      ARGS_GROUP_COLUMNS,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
-    },
-    {
-      "clumpsarea",
-      UI_KEY_CLUMPSAREA,
-      0,
-      0,
-      "Sum of all clump areas in an object.",
-      ARGS_GROUP_COLUMNS,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
-    },
-    {
-      "weightarea",
-      UI_KEY_WEIGHTAREA,
-      0,
-      0,
-      "Area used for flux weighted positions.",
-      ARGS_GROUP_COLUMNS,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
+      0, 0, 0, 0,
+      "Positional columns (pixel)",
+      UI_GROUP_COLUMNS_POSITION_PIXEL
     },
     {
       "x",
@@ -423,7 +432,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Flux weighted center in first FITS axis.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_POSITION_PIXEL,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -437,7 +446,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Flux weighted center in second FITS axis.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_POSITION_PIXEL,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -451,7 +460,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Geometric center in first FITS axis.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_POSITION_PIXEL,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -465,7 +474,63 @@ struct argp_option program_options[] =
       0,
       0,
       "Geometric center in second FITS axis.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_POSITION_PIXEL,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "minx",
+      UI_KEY_MINX,
+      0,
+      0,
+      "Minimum first FITS axis position.",
+      UI_GROUP_COLUMNS_POSITION_PIXEL,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "maxx",
+      UI_KEY_MAXX,
+      0,
+      0,
+      "Maximum first FITS axis position.",
+      UI_GROUP_COLUMNS_POSITION_PIXEL,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "miny",
+      UI_KEY_MINY,
+      0,
+      0,
+      "Minimum second FITS axis position.",
+      UI_GROUP_COLUMNS_POSITION_PIXEL,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "maxy",
+      UI_KEY_MAXY,
+      0,
+      0,
+      "Maximum second FITS axis position.",
+      UI_GROUP_COLUMNS_POSITION_PIXEL,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -479,7 +544,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Flux.wht center of all clumps in obj. (X).",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_POSITION_PIXEL,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -493,7 +558,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Flux.wht center of all clumps in obj. (Y).",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_POSITION_PIXEL,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -507,7 +572,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Geometric center of all clumps in obj. (X).",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_POSITION_PIXEL,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -521,7 +586,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Geometric center of all clumps in obj. (Y).",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_POSITION_PIXEL,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -529,13 +594,24 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       ui_column_codes_ll
     },
+
+
+
+
+
+    /* Position related columns (WCS). */
+    {
+      0, 0, 0, 0,
+      "Positional columns (WCS)",
+      UI_GROUP_COLUMNS_POSITION_WCS
+    },
     {
       "ra",
       UI_KEY_RA,
       0,
       0,
-      "Right ascension of flux weighted center.",
-      ARGS_GROUP_COLUMNS,
+      "Flux weighted center right ascension.",
+      UI_GROUP_COLUMNS_POSITION_WCS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -548,8 +624,8 @@ struct argp_option program_options[] =
       UI_KEY_DEC,
       0,
       0,
-      "Declination of flux weighted center.",
-      ARGS_GROUP_COLUMNS,
+      "Flux weighted center declination.",
+      UI_GROUP_COLUMNS_POSITION_WCS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -558,12 +634,12 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "geora",
-      UI_KEY_GEORA,
+      "w1",
+      UI_KEY_W1,
       0,
       0,
-      "Right ascension of geometric center.",
-      ARGS_GROUP_COLUMNS,
+      "Flux weighted center in first WCS axis.",
+      UI_GROUP_COLUMNS_POSITION_WCS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -572,12 +648,12 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "geodec",
-      UI_KEY_GEODEC,
+      "w2",
+      UI_KEY_W2,
       0,
       0,
-      "Declination of geometric center.",
-      ARGS_GROUP_COLUMNS,
+      "Flux weighted center in second WCS axis.",
+      UI_GROUP_COLUMNS_POSITION_WCS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -586,12 +662,12 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsra",
-      UI_KEY_CLUMPSRA,
+      "geow1",
+      UI_KEY_GEOW1,
       0,
       0,
-      "Right ascension of f.wht center of all clumps.",
-      ARGS_GROUP_COLUMNS,
+      "Geometric center in first WCS axis.",
+      UI_GROUP_COLUMNS_POSITION_WCS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -600,12 +676,12 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsdec",
-      UI_KEY_CLUMPSDEC,
+      "geow2",
+      UI_KEY_GEOW2,
       0,
       0,
-      "Declination of f.wht center of all clumps.",
-      ARGS_GROUP_COLUMNS,
+      "Geometric center in second WCS axis.",
+      UI_GROUP_COLUMNS_POSITION_WCS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -614,12 +690,12 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsgeora",
-      UI_KEY_CLUMPSGEORA,
+      "clumpsw1",
+      UI_KEY_CLUMPSW1,
       0,
       0,
-      "Right ascension of geo. center of all clumps.",
-      ARGS_GROUP_COLUMNS,
+      "Flux.wht center of all clumps in 1st WCS.",
+      UI_GROUP_COLUMNS_POSITION_WCS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -628,12 +704,12 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsgeodec",
-      UI_KEY_CLUMPSGEODEC,
+      "clumpsw2",
+      UI_KEY_CLUMPSW2,
       0,
       0,
-      "Declination of geometric center of all clumps.",
-      ARGS_GROUP_COLUMNS,
+      "Flux.wht center of all clumps in 2nd WCS.",
+      UI_GROUP_COLUMNS_POSITION_WCS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -641,13 +717,66 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       ui_column_codes_ll
     },
+    {
+      "clumpsgeow1",
+      UI_KEY_CLUMPSGEOW1,
+      0,
+      0,
+      "Geometric center of all clumps in 1st WCS.",
+      UI_GROUP_COLUMNS_POSITION_WCS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "clumpsgeow2",
+      UI_KEY_CLUMPSGEOW2,
+      0,
+      0,
+      "Geometric center of all clumps in 2nd WCS.",
+      UI_GROUP_COLUMNS_POSITION_WCS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+
+
+
+
+
+    /* Brightness/pixel-value related columns. */
+    {
+      0, 0, 0, 0,
+      "Brightness/magnitude related columns",
+      UI_GROUP_COLUMNS_BRIGHTNESS
+    },
     {
       "brightness",
       UI_KEY_BRIGHTNESS,
       0,
       0,
       "Brightness (sum of pixel values).",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_BRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "brightnesserr",
+      UI_KEY_BRIGHTNESSERR,
+      0,
+      0,
+      "Error (1-sigma) in measuring brightness.",
+      UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -661,7 +790,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Brightness of clumps in an object.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -670,12 +799,40 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "noriverbrightness",
-      UI_KEY_NORIVERBRIGHTNESS,
+      "brightnessnoriver",
+      UI_KEY_BRIGHTNESSNORIVER,
       0,
       0,
       "Sky (not river) subtracted clump brightness.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_BRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "mean",
+      UI_KEY_MEAN,
+      0,
+      0,
+      "Mean of values in object/clump.",
+      UI_GROUP_COLUMNS_BRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "median",
+      UI_KEY_MEDIAN,
+      0,
+      0,
+      "Median of values in object/clump.",
+      UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -689,7 +846,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Total magnitude of objects or clumps.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -703,7 +860,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Magnitude error of objects or clumps.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -717,7 +874,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Magnitude of all clumps in object.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -731,7 +888,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Upper-limit value, use other options to config.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -745,7 +902,63 @@ struct argp_option program_options[] =
       0,
       0,
       "Upper-limit mag. use other options to config.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_BRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "upperlimitonesigma",
+      UI_KEY_UPPERLIMITONESIGMA,
+      0,
+      0,
+      "Upper-limit one sigma value.",
+      UI_GROUP_COLUMNS_BRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "upperlimitsigma",
+      UI_KEY_UPPERLIMITSIGMA,
+      0,
+      0,
+      "Place in random distribution (sigma multiple).",
+      UI_GROUP_COLUMNS_BRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "upperlimitquantile",
+      UI_KEY_UPPERLIMITQUANTILE,
+      0,
+      0,
+      "Quantile in random distribution (max 1).",
+      UI_GROUP_COLUMNS_BRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "upperlimitskew",
+      UI_KEY_UPPERLIMITSKEW,
+      0,
+      0,
+      "(Mean-Median)/STD of random distribution.",
+      UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -759,7 +972,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Average river value surrounding a clump.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -773,7 +986,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Number of river pixels around a clump.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -787,7 +1000,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Signal to noise ratio of objects or clumps.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -801,7 +1014,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Average Sky value under this clump or object.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -814,8 +1027,88 @@ struct argp_option program_options[] =
       UI_KEY_STD,
       0,
       0,
-      "Average Sky standard deviation.",
-      ARGS_GROUP_COLUMNS,
+      "Average Sky standard deviation under label.",
+      UI_GROUP_COLUMNS_BRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+
+
+
+
+    /* Morphology/shape related columns. */
+    {
+      0, 0, 0, 0,
+      "Morphology/shape related columns",
+      UI_GROUP_COLUMNS_MORPHOLOGY
+    },
+    {
+      "numclumps",
+      UI_KEY_NUMCLUMPS,
+      0,
+      0,
+      "Number of clumps in this object.",
+      UI_GROUP_COLUMNS_MORPHOLOGY,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "area",
+      UI_KEY_AREA,
+      0,
+      0,
+      "Number of non-blank valued pixels.",
+      UI_GROUP_COLUMNS_MORPHOLOGY,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "clumpsarea",
+      UI_KEY_CLUMPSAREA,
+      0,
+      0,
+      "Non-blank area covered by clumps.",
+      UI_GROUP_COLUMNS_MORPHOLOGY,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "weightarea",
+      UI_KEY_WEIGHTAREA,
+      0,
+      0,
+      "Area used for value weighted positions.",
+      UI_GROUP_COLUMNS_MORPHOLOGY,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "geoarea",
+      UI_KEY_GEOAREA,
+      0,
+      0,
+      "Area labled region (irrespective of value).",
+      UI_GROUP_COLUMNS_MORPHOLOGY,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -829,7 +1122,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Flux weighted semi-major axis.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_MORPHOLOGY,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -843,7 +1136,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Flux weighted semi-minor axis.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_MORPHOLOGY,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -857,7 +1150,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Flux weighted axis ratio.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_MORPHOLOGY,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -871,7 +1164,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Flux weighted position angle.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_MORPHOLOGY,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -885,7 +1178,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Geometric semi-major axis.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_MORPHOLOGY,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -899,7 +1192,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Geometric semi-minor axis.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_MORPHOLOGY,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -913,7 +1206,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Geometric axis ratio.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_MORPHOLOGY,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
@@ -927,7 +1220,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Geometric position angle.",
-      ARGS_GROUP_COLUMNS,
+      UI_GROUP_COLUMNS_MORPHOLOGY,
       0,
       GAL_TYPE_INVALID,
       GAL_OPTIONS_RANGE_ANY,
