@@ -610,52 +610,51 @@ mkcatalog_write_outputs(struct mkcatalogparams *p)
   gal_list_str_t *comments;
 
 
-  /* OBJECT CATALOG
-     ============== */
+  /* OBJECT catalog */
   comments=mkcatalog_outputs_same_start(p, 0, "Detection");
 
-
-  /* Write objects catalog
-     ---------------------
-
-     Reverse the comments list (so it is printed in the same order here),
+  /* Reverse the comments list (so it is printed in the same order here),
      write the objects catalog and free the comments. */
   gal_list_str_reverse(&comments);
-  gal_table_write(p->objectcols, comments, p->cp.tableformat, p->objectsout,
-                  "OBJECTS", 0);
+  gal_table_write(p->objectcols, comments, p->cp.tableformat,
+                  p->objectsout, "OBJECTS", 0);
   gal_list_str_free(comments, 1);
 
 
-
-  /* CLUMPS CATALOG
-     ============== */
+  /* CLUMPS catalog */
   if(p->clumps)
     {
       /* Make the comments. */
       comments=mkcatalog_outputs_same_start(p, 1, "Clumps");
 
-      /* Write objects catalog
-         ---------------------
-
-         Reverse the comments list (so it is printed in the same order here),
-         write the objects catalog and free the comments. */
+      /* Reverse the comments and write the catalog. */
       gal_list_str_reverse(&comments);
-      gal_table_write(p->clumpcols, comments, p->cp.tableformat, p->clumpsout,
-                      "CLUMPS", 0);
+      gal_table_write(p->clumpcols, comments, p->cp.tableformat,
+                      p->clumpsout, "CLUMPS", 0);
       gal_list_str_free(comments, 1);
     }
 
-  /* Inform the user. */
+
+  /* Configuration information. */
+  if(gal_fits_name_is_fits(p->objectsout))
+    {
+      gal_fits_key_write_filename("input", p->objectsfile, &p->cp.okeys, 1);
+      gal_fits_key_write_config(&p->cp.okeys, "MakeCatalog configuration",
+                                "MKCATALOG-CONFIG", p->objectsout, "0");
+    }
+
+
+  /* Inform the user */
   if(!p->cp.quiet)
     {
-      if(p->clumpsout==p->objectsout)
-        printf("  - Output catalog: %s\n", p->objectsout);
-      else
+      if(p->clumpsout && strcmp(p->clumpsout,p->objectsout))
         {
           printf("  - Output objects catalog: %s\n", p->objectsout);
           if(p->clumps)
             printf("  - Output clumps catalog: %s\n", p->clumpsout);
         }
+      else
+        printf("  - Catalog written to %s\n", p->objectsout);
     }
 }
 
