@@ -54,7 +54,7 @@ enum option_keys_enum
   /* With short-option version. */
   UI_KEY_BACKGROUND      = 'k',
   UI_KEY_BACKHDU         = 'B',
-  UI_KEY_NAXIS           = 'x',
+  UI_KEY_MERGEDSIZE      = 'x',
   UI_KEY_CLEARCANVAS     = 'C',
   UI_KEY_KERNEL          = 'E',
   UI_KEY_OVERSAMPLE      = 's',
@@ -99,6 +99,9 @@ enum option_keys_enum
 void
 ui_read_check_inputs_setup(int argc, char *argv[], struct mkprofparams *p);
 
+char *
+ui_profile_name_write(int profile_code);
+
 void
 ui_free_report(struct mkprofparams *p, struct timeval *t1);
 
