@@ -3,9 +3,9 @@ Table - View and manipulate a FITS table structures.
 Table is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016, Free Software Foundation, Inc.
+Copyright (C) 2016-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -120,11 +120,17 @@ ui_initialize_options(struct tableparams *p,
       /* Select individually. */
       switch(cp->coptions[i].key)
         {
+        /* Mandatory options. */
         case GAL_OPTIONS_KEY_SEARCHIN:
         case GAL_OPTIONS_KEY_MINMAPSIZE:
         case GAL_OPTIONS_KEY_TABLEFORMAT:
           cp->coptions[i].mandatory=GAL_OPTIONS_MANDATORY;
           break;
+
+        /* Options to ignore. */
+        case GAL_OPTIONS_KEY_TYPE:
+          cp->coptions[i].flags=OPTION_HIDDEN;
+          break;
         }
 
       /* Select by group. */
@@ -236,8 +242,6 @@ ui_check_options_and_arguments(struct tableparams *p)
               "zero), extension name, or anything acceptable by CFITSIO");
 
     }
-  else
-    error(EXIT_FAILURE, 0, "no input file is specified");
 }
 
 
@@ -262,17 +266,21 @@ ui_check_options_and_arguments(struct tableparams *p)
 /**************************************************************/
 /***************       Preparations         *******************/
 /**************************************************************/
-void
+static void
 ui_print_info_exit(struct tableparams *p)
 {
   char *tmp;
   int tableformat;
   gal_data_t *allcols;
+  gal_list_str_t *lines;
   size_t i, numcols, numrows;
 
   /* Read the table information for the number of columns and rows. */
-  allcols=gal_table_info(p->filename, p->cp.hdu, &numcols,
+  lines=gal_options_check_stdin(p->filename, p->cp.stdintimeout, "input");
+  allcols=gal_table_info(p->filename, p->cp.hdu, lines, &numcols,
                          &numrows, &tableformat);
+  if(p->filename==NULL) p->filename="Standard-input";
+  gal_list_str_free(lines, 1);
 
   /* If there was no actual data in the file, then inform the user */
   if(allcols==NULL)
@@ -307,9 +315,54 @@ ui_print_info_exit(struct tableparams *p)
 
 
 
-void
+/* The columns can be given as comma-separated values to one option or
+   multiple calls to the column option. Here, we'll check if the input list
+   has comma-separated values. If they do then the list will be updated to
+   be fully separate. */
+static void
+ui_columns_prepare(struct tableparams *p)
+{
+  size_t i;
+  char **strarr;
+  gal_data_t *strs;
+  gal_list_str_t *tmp, *new=NULL;
+
+  /* Go over the whole original list (where each node may have more than
+     one value separated by a comma. */
+  for(tmp=p->columns;tmp!=NULL;tmp=tmp->next)
+    {
+      /* Read the different comma-separated strings into an array (within a
+         `gal_data_t'). */
+      strs=gal_options_parse_csv_strings_raw(tmp->v, NULL, 0);
+      strarr=strs->array;
+
+      /* Go over all the elements and add them to the `new' list. */
+      for(i=0;i<strs->size;++i)
+        {
+          gal_list_str_add(&new, strarr[i], 0);
+          strarr[i]=NULL;
+        }
+
+      /* Clean up. */
+      gal_data_free(strs);
+    }
+
+  /* Delete the old list. */
+  gal_list_str_free(p->columns, 1);
+
+  /* Reverse the new list, then put it into `p->columns'. */
+  gal_list_str_reverse(&new);
+  p->columns=new;
+}
+
+
+
+
+
+static void
 ui_preparations(struct tableparams *p)
 {
+  gal_list_str_t *lines;
   struct gal_options_common_params *cp=&p->cp;
 
   /* If there were no columns specified or the user has asked for
@@ -317,9 +370,16 @@ ui_preparations(struct tableparams *p)
   if(p->information)
     ui_print_info_exit(p);
 
+  /* Prepare the column names. */
+  ui_columns_prepare(p);
+
   /* Read in the table columns. */
-  p->table=gal_table_read(p->filename, cp->hdu, p->columns, cp->searchin,
-                          cp->ignorecase, cp->minmapsize, NULL);
+  lines=gal_options_check_stdin(p->filename, p->cp.stdintimeout, "input");
+  p->table=gal_table_read(p->filename, cp->hdu, lines, p->columns,
+                          cp->searchin, cp->ignorecase, cp->minmapsize,
+                          NULL);
+  if(p->filename==NULL) p->filename="Standard-input";
+  gal_list_str_free(lines, 1);
 
   /* If there was no actual data in the file, then inform the user and
      abort. */
