@@ -3,9 +3,9 @@ Warp - Warp images using projective mapping.
 Warp is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -111,8 +111,8 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /***************************************************************/
 /**************      Processing function      ******************/
 /***************************************************************/
-void *
-warponthread(void *inparam)
+static void *
+warp_onthread(void *inparam)
 {
   struct iwpparams *iwp=(struct iwpparams*)inparam;
   struct warpparams *p=iwp->p;
@@ -291,8 +291,8 @@ warponthread(void *inparam)
    array to the input array. The order is fixed for all the pixels in
    the image altough the scale might change.
 */
-void
-warppreparations(struct warpparams *p)
+static void
+warp_preparations(struct warpparams *p)
 {
   double is0=p->input->dsize[0], is1=p->input->dsize[1];
 
@@ -376,8 +376,6 @@ warppreparations(struct warpparams *p)
   p->opixarea=gal_polygon_area(forarea, 4);
 
 
-
-
   /* Find which index after transformation will have the minimum and
      maximum positions along the two axises. We can't use the starting
      loop because that is based on the input image which can be
@@ -405,42 +403,64 @@ warppreparations(struct warpparams *p)
 
 
 /* Correct the WCS coordinates (Multiply the 2x2 PC matrix of the WCS
-   structure by the INVERSE of the transform in 2x2 form that has been
-   converted from homogeneous coordinates). Then Multiply the crpix
-   array with the ACTUAL transformation matrix. */
+   structure by the INVERSE of the transform in 2x2). Then Multiply the
+   crpix array with the ACTUAL transformation matrix. */
 void
 correct_wcs_save_output(struct warpparams *p)
 {
   size_t i;
-  double *m=p->matrix->array, diff;
+  double tcrpix[3];
   char keyword[9*FLEN_KEYWORD];
+  double *m=p->matrix->array, diff;
   struct wcsprm *wcs=p->output->wcs;
   gal_fits_list_key_t *headers=NULL;
-  double tpc[4], tcrpix[3], *pixelscale;
-  double *crpix=wcs->crpix, *pc=wcs->pc;
+  double *crpix=wcs?wcs->crpix:NULL, *w=p->inwcsmatrix;
+
+  /* `tinv' is the 2 by 2 inverse matrix. Recall that `p->inverse' is 3 by
+     3 to account for homogeneous coordinates. */
   double tinv[4]={p->inverse[0]/p->inverse[8], p->inverse[1]/p->inverse[8],
                   p->inverse[3]/p->inverse[8], p->inverse[4]/p->inverse[8]};
 
-  if(p->keepwcs==0 && wcs)
+  /* Make the WCS corrections if necessary. */
+  if(wcs)
     {
-      /* Correct the PC matrix: */
-      tpc[0]=pc[0]*tinv[0]+pc[1]*tinv[2];
-      tpc[1]=pc[0]*tinv[1]+pc[1]*tinv[3];
-      tpc[2]=pc[2]*tinv[0]+pc[3]*tinv[2];
-      tpc[3]=pc[2]*tinv[1]+pc[3]*tinv[3];
-      pc[0]=tpc[0]; pc[1]=tpc[1]; pc[2]=tpc[2]; pc[3]=tpc[3];
+      if(p->keepwcs==0)
+        {
+          /* Correct the input WCS matrix. Since we are re-writing the PC
+             matrix from the full rotation matrix (including pixel scale),
+             we'll also have to set the CDELT fields to 1. Just to be sure
+             that the PC matrix is used in the end by WCSLIB, we'll also
+             set altlin to 1.*/
+          wcs->altlin=1;
+          wcs->cdelt[0] = wcs->cdelt[1] = 1.0f;
+          wcs->pc[0] = w[0]*tinv[0] + w[1]*tinv[2];
+          wcs->pc[1] = w[0]*tinv[1] + w[1]*tinv[3];
+          wcs->pc[2] = w[2]*tinv[0] + w[3]*tinv[2];
+          wcs->pc[3] = w[2]*tinv[1] + w[3]*tinv[3];
 
-      /* Correct the CRPIX point. The +1 in the end of the last two
-         lines is because FITS counts from 1. */
-      tcrpix[0]=m[0]*crpix[0]+m[1]*crpix[1]+m[2];
-      tcrpix[1]=m[3]*crpix[0]+m[4]*crpix[1]+m[5];
-      tcrpix[2]=m[6]*crpix[0]+m[7]*crpix[1]+m[8];
-      crpix[0]=tcrpix[0]/tcrpix[2]-p->outfpixval[0]+1;
-      crpix[1]=tcrpix[1]/tcrpix[2]-p->outfpixval[1]+1;
+          /* Correct the CRPIX point. The +1 in the end of the last two
+             lines is because FITS counts from 1. */
+          tcrpix[0] = m[0]*crpix[0]+m[1]*crpix[1]+m[2];
+          tcrpix[1] = m[3]*crpix[0]+m[4]*crpix[1]+m[5];
+          tcrpix[2] = m[6]*crpix[0]+m[7]*crpix[1]+m[8];
+
+          crpix[0] = tcrpix[0]/tcrpix[2] - p->outfpixval[0] + 1;
+          crpix[1] = tcrpix[1]/tcrpix[2] - p->outfpixval[1] + 1;
+        }
+
+      /* Due to floating point errors extremely small values of PC matrix
+         can be set to zero and extremely small differences between PC1_1
+         and PC2_2 can be ignored. The reason for all the `fabs' functions
+         is because the signs are usually different.*/
+      if( fabs(wcs->pc[1])<ABSOLUTEFLTERROR ) wcs->pc[1]=0.0f;
+      if( fabs(wcs->pc[2])<ABSOLUTEFLTERROR ) wcs->pc[2]=0.0f;
+      diff=fabs(wcs->pc[0])-fabs(wcs->pc[3]);
+      if( fabs(diff/p->pixelscale[0])<RELATIVEFLTERROR )
+        wcs->pc[3]=( (wcs->pc[3] < 0.0f ? -1.0f : 1.0f) * fabs(wcs->pc[0]) );
     }
 
   /* Add the appropriate headers: */
-  gal_fits_key_write_filename("INF", p->inputname, &headers);
+  gal_fits_key_write_filename("INF", p->inputname, &headers, 0);
   for(i=0;i<9;++i)
     {
       sprintf(&keyword[i*FLEN_KEYWORD], "WMTX%zu_%zu", i/3+1, i%3+1);
@@ -449,24 +469,15 @@ correct_wcs_save_output(struct warpparams *p)
                                 "Warp matrix element value", 0, NULL);
     }
 
-  /* Due to floating point errors extremely small values of PC matrix can
-     be set to zero and extremely small differences between PC1_1 and PC2_2
-     can be ignored. The reason for all the `fabs' functions is because the
-     signs are usually different.*/
-  if( wcs->pc[1]<ABSOLUTEFLTERROR ) wcs->pc[1]=0.0f;
-  if( wcs->pc[2]<ABSOLUTEFLTERROR ) wcs->pc[2]=0.0f;
-  pixelscale=gal_wcs_pixel_scale_deg(wcs);
-  diff=fabs(wcs->pc[0])-fabs(wcs->pc[3]);
-  if( fabs(diff/pixelscale[0])<RELATIVEFLTERROR )
-    wcs->pc[3] =  ( (wcs->pc[3] < 0.0f ? -1.0f : 1.0f) * fabs(wcs->pc[0]) );
-
   /* Save the output into the proper type and write it. */
-  if(p->cp.type!=p->output->type)
+  if(p->cp.type && p->cp.type!=p->output->type)
     p->output=gal_data_copy_to_new_type_free(p->output, p->cp.type);
-  gal_fits_img_write(p->output, p->cp.output, headers, PROGRAM_STRING);
+  gal_fits_img_write(p->output, p->cp.output, headers, PROGRAM_NAME);
 
-  /* Clean up. */
-  free(pixelscale);
+  /* Write the configuration keywords. */
+  gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1);
+  gal_fits_key_write_config(&p->cp.okeys, "Warp configuration",
+                            "WARP-CONFIG", p->cp.output, "0");
 }
 
 
@@ -512,19 +523,19 @@ warp(struct warpparams *p)
 
 
   /* Prepare the output array and all the necessary things: */
-  warppreparations(p);
+  warp_preparations(p);
 
 
   /* Distribute the output pixels into the threads: */
   gal_threads_dist_in_threads(p->output->size, nt, &indexs, &thrdcols);
 
 
-  /* Start the convolution. */
+  /* Start the warp. */
   if(nt==1)
     {
       iwp[0].p=p;
       iwp[0].indexs=indexs;
-      warponthread(&iwp[0]);
+      warp_onthread(&iwp[0]);
     }
   else
     {
@@ -543,7 +554,7 @@ warp(struct warpparams *p)
             iwp[i].p=p;
             iwp[i].b=&b;
             iwp[i].indexs=&indexs[i*thrdcols];
-            err=pthread_create(&t, &attr, warponthread, &iwp[i]);
+            err=pthread_create(&t, &attr, warp_onthread, &iwp[i]);
             if(err)
               error(EXIT_FAILURE, 0, "%s: can't create thread %zu",
                     __func__, i);
