@@ -3,9 +3,9 @@ blank -- Deal with blank values in a datasets
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017, Free Software Foundation, Inc.
+Copyright (C) 2017-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -32,6 +32,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/data.h>
 #include <gnuastro/tile.h>
 #include <gnuastro/blank.h>
+#include <gnuastro/pointer.h>
 
 #include <gnuastro-internal/checkset.h>
 
@@ -87,7 +88,7 @@ gal_blank_alloc_write(uint8_t type)
   void *out;
 
   /* Allocate the space to keep the blank value. */
-  out=gal_data_malloc_array(type, 1, __func__, "out");
+  out=gal_pointer_allocate(type, 1, 0, __func__, "out");
 
   /* Put the blank value in the allcated space. */
   gal_blank_write(out, type);
@@ -112,6 +113,235 @@ gal_blank_initialize(gal_data_t *input)
 
 
 
+/* Print the blank value as a string. For the integer types, we'll use the
+   PRIxNN keywords of `inttypes.h' (which is imported into Gnuastro from
+   Gnulib, so we don't necessarily rely on the host system having it). */
+char *
+gal_blank_as_string(uint8_t type, int width)
+{
+  char *blank=NULL, *fmt;
+
+  /* Print the given value. */
+  switch(type)
+    {
+    case GAL_TYPE_BIT:
+      error(EXIT_FAILURE, 0, "%s: bit types are not implemented yet",
+            __func__);
+      break;
+
+    case GAL_TYPE_STRING:
+      if(width)
+        {
+          if( asprintf(&blank, "%*s", width,  GAL_BLANK_STRING)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, "%s", GAL_BLANK_STRING)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      break;
+
+    case GAL_TYPE_UINT8:
+      fmt = width ? "%*"PRIu8 : "%"PRIu8;
+      if(width)
+        {
+          if( asprintf(&blank, fmt, width, (uint8_t)GAL_BLANK_UINT8)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, fmt, (uint8_t)GAL_BLANK_UINT8)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      break;
+
+    case GAL_TYPE_INT8:
+      fmt = width ? "%*"PRId8 : "%"PRId8;
+      if(width)
+        {
+          if( asprintf(&blank, fmt, width, (int8_t)GAL_BLANK_INT8)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, fmt, (int8_t)GAL_BLANK_INT8)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      break;
+
+    case GAL_TYPE_UINT16:
+      fmt = width ? "%*"PRIu16 : "%"PRIu16;
+      if(width)
+        {
+          if( asprintf(&blank, fmt, width, (uint16_t)GAL_BLANK_UINT16)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, fmt, (uint16_t)GAL_BLANK_UINT16)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      break;
+
+    case GAL_TYPE_INT16:
+      fmt = width ? "%*"PRId16 : "%"PRId16;
+      if(width)
+        {
+          if( asprintf(&blank, fmt, width, (int16_t)GAL_BLANK_INT16)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, fmt, (int16_t)GAL_BLANK_INT16)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      break;
+
+    case GAL_TYPE_UINT32:
+      fmt = width ? "%*"PRIu32 : "%"PRIu32;
+      if(width)
+        {
+          if( asprintf(&blank, fmt, width, (uint32_t)GAL_BLANK_UINT32)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, fmt, (uint32_t)GAL_BLANK_UINT32)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      break;
+
+    case GAL_TYPE_INT32:
+      fmt = width ? "%*"PRId32 : "%"PRId32;
+      if(width)
+        {
+          if( asprintf(&blank, fmt, width, (int32_t)GAL_BLANK_INT32)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, fmt, (int32_t)GAL_BLANK_INT32)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      break;
+
+    case GAL_TYPE_UINT64:
+      fmt = width ? "%*"PRIu64 : "%"PRIu64;
+      if(width)
+        {
+          if( asprintf(&blank, fmt, width, (uint64_t)GAL_BLANK_UINT64)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, fmt, (uint64_t)GAL_BLANK_UINT64)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      break;
+
+    case GAL_TYPE_INT64:
+      fmt = width ? "%*"PRId64 : "%"PRId64;
+      if(width)
+        {
+          if( asprintf(&blank, fmt, width, (int64_t)GAL_BLANK_INT64)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, fmt, (int64_t)GAL_BLANK_INT64)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      break;
+
+    case GAL_TYPE_FLOAT32:
+      if(width)
+        {
+          if( asprintf(&blank, "%*f", width,  (float)GAL_BLANK_FLOAT32)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, "%f", (float)GAL_BLANK_FLOAT32)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      break;
+
+    case GAL_TYPE_FLOAT64:
+      if(width)
+        {
+          if( asprintf(&blank, "%*f", width,  (double)GAL_BLANK_FLOAT64)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, "%f", (double)GAL_BLANK_FLOAT64)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      break;
+
+    default:
+      error(EXIT_FAILURE, 0, "%s: type code %d not recognized", __func__,
+            type);
+    }
+  return blank;
+}
+
+
+
+
+
+
+/* Return 1 if the contents of the pointer (with the given type) is
+   blank. */
+int
+gal_blank_is(void *pointer, uint8_t type)
+{
+  switch(type)
+    {
+    /* Numeric types */
+    case GAL_TYPE_UINT8:     return *(uint8_t  *)pointer==GAL_BLANK_UINT8;
+    case GAL_TYPE_INT8:      return *(int8_t   *)pointer==GAL_BLANK_INT8;
+    case GAL_TYPE_UINT16:    return *(uint16_t *)pointer==GAL_BLANK_UINT16;
+    case GAL_TYPE_INT16:     return *(int16_t  *)pointer==GAL_BLANK_INT16;
+    case GAL_TYPE_UINT32:    return *(uint32_t *)pointer==GAL_BLANK_UINT32;
+    case GAL_TYPE_INT32:     return *(int32_t  *)pointer==GAL_BLANK_INT32;
+    case GAL_TYPE_UINT64:    return *(uint64_t *)pointer==GAL_BLANK_UINT64;
+    case GAL_TYPE_INT64:     return *(int64_t  *)pointer==GAL_BLANK_INT64;
+    case GAL_TYPE_FLOAT32:   return isnan( *(float *)(pointer) );
+    case GAL_TYPE_FLOAT64:   return isnan( *(double *)(pointer) );
+
+    /* String. */
+    case GAL_TYPE_STRING:    if(!strcmp(pointer,GAL_BLANK_STRING)) return 1;
+
+    /* Complex types */
+    case GAL_TYPE_COMPLEX32:
+    case GAL_TYPE_COMPLEX64:
+      error(EXIT_FAILURE, 0, "%s: complex types are not yet supported",
+            __func__);
+
+    /* Bit. */
+    case GAL_TYPE_BIT:
+      error(EXIT_FAILURE, 0, "%s: bit type datasets are not yet supported",
+            __func__);
+
+    default:
+      error(EXIT_FAILURE, 0, "%s: type value (%d) not recognized",
+            __func__, type);
+    }
+
+  /* Control should not reach here, so print an error if it does, then
+     return a 0 (just to avoid compiler warnings). */
+  error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to address the "
+        "problem. Control should not reach the end of this function",
+        __func__, PACKAGE_BUGREPORT);
+  return 0;
+}
+
+
+
+
+
 /* Return 1 if the dataset has a blank value and zero if it doesn't. Before
    checking the dataset, this function will look at its flags. If the
    `GAL_DATA_FLAG_HASBLANK' or `GAL_DATA_FLAG_DONT_CHECK_ZERO' bits of
@@ -168,10 +398,10 @@ gal_blank_present(gal_data_t *input, int updateflag)
      blank is present. */
   if(input->size==0) return 0;
 
-  /* From the user's flags, it might not be necessary to go through the
-     dataset any more.  */
-  if( ( input->flag & GAL_DATA_FLAG_HASBLANK )
-      || ( input->flag & GAL_DATA_FLAG_BLANK_CH ) )
+  /* From the user's flags, you can tell if the dataset has already been
+     checked for blank values or not. If it has, then just return the
+     checked result. */
+  if( input->flag & GAL_DATA_FLAG_BLANK_CH )
     return input->flag & GAL_DATA_FLAG_HASBLANK;
 
   /* Go over the pixels and check: */
@@ -195,7 +425,7 @@ gal_blank_present(gal_data_t *input, int updateflag)
         error(EXIT_FAILURE, 0, "%s: tile mode is currently not supported for "
               "strings", __func__);
       strf = (str=input->array) + input->size;
-      do if(!strcmp(*str++,GAL_BLANK_STRING)) return 1; while(str<strf);
+      do if(!strcmp(*str,GAL_BLANK_STRING)) return 1; while(++str<strf);
       break;
 
     /* Complex types */
@@ -266,7 +496,7 @@ gal_blank_flag(gal_data_t *input)
       /* Go over the pixels and set the output values. */
       switch(input->type)
         {
-          /* Numeric types */
+        /* Numeric types */
         case GAL_TYPE_UINT8:     FLAG_BLANK( uint8_t  );    break;
         case GAL_TYPE_INT8:      FLAG_BLANK( int8_t   );    break;
         case GAL_TYPE_UINT16:    FLAG_BLANK( uint16_t );    break;
@@ -278,19 +508,19 @@ gal_blank_flag(gal_data_t *input)
         case GAL_TYPE_FLOAT32:   FLAG_BLANK( float    );    break;
         case GAL_TYPE_FLOAT64:   FLAG_BLANK( double   );    break;
 
-          /* String. */
+        /* String. */
         case GAL_TYPE_STRING:
           do *o++ = !strcmp(*str,GAL_BLANK_STRING); while(++str<strf);
           break;
 
-          /* Currently unsupported types. */
+        /* Currently unsupported types. */
         case GAL_TYPE_BIT:
         case GAL_TYPE_COMPLEX32:
         case GAL_TYPE_COMPLEX64:
           error(EXIT_FAILURE, 0, "%s: %s type not yet supported",
                 __func__, gal_type_name(input->type, 1));
 
-          /* Bad input. */
+        /* Bad input. */
         default:
           error(EXIT_FAILURE, 0, "%s: type value (%d) not recognized",
                 __func__, input->type);
@@ -309,6 +539,78 @@ gal_blank_flag(gal_data_t *input)
 
 
 
+/* Write a blank value in the input anywhere that the flag dataset is not
+   zero or not blank. */
+#define BLANK_FLAG_APPLY(IT) {                                          \
+    IT *i=input->array, b;                                              \
+    gal_blank_write(&b, input->type);                                   \
+    do {if(*f && *f!=GAL_BLANK_UINT8) *i=b; ++i;} while(++f<ff);        \
+  }
+void
+gal_blank_flag_apply(gal_data_t *input, gal_data_t *flag)
+{
+  char **str=input->array;
+  uint8_t *f=flag->array, *ff=f+flag->size;
+
+  /* Sanity check. */
+  if(flag->type!=GAL_TYPE_UINT8)
+    error(EXIT_FAILURE, 0, "%s: the `flag' argument has a `%s' type, it "
+          "must have an unsigned 8-bit type", __func__,
+          gal_type_name(flag->type, 1));
+  if(gal_dimension_is_different(input, flag))
+    error(EXIT_FAILURE, 0, "%s: the `flag' argument doesn't have the same "
+          "size as the `input' argument", __func__);
+
+  /* Write the blank values. */
+  switch(input->type)
+    {
+    /* Basic numeric types. */
+    case GAL_TYPE_UINT8:     BLANK_FLAG_APPLY( uint8_t  );    break;
+    case GAL_TYPE_INT8:      BLANK_FLAG_APPLY( int8_t   );    break;
+    case GAL_TYPE_UINT16:    BLANK_FLAG_APPLY( uint16_t );    break;
+    case GAL_TYPE_INT16:     BLANK_FLAG_APPLY( int16_t  );    break;
+    case GAL_TYPE_UINT32:    BLANK_FLAG_APPLY( uint32_t );    break;
+    case GAL_TYPE_INT32:     BLANK_FLAG_APPLY( int32_t  );    break;
+    case GAL_TYPE_UINT64:    BLANK_FLAG_APPLY( uint64_t );    break;
+    case GAL_TYPE_INT64:     BLANK_FLAG_APPLY( int64_t  );    break;
+    case GAL_TYPE_FLOAT32:   BLANK_FLAG_APPLY( float    );    break;
+    case GAL_TYPE_FLOAT64:   BLANK_FLAG_APPLY( double   );    break;
+
+    /* Strings. */
+    case GAL_TYPE_STRING:
+      do
+        {
+          if(*f && *f!=GAL_BLANK_UINT8)
+            {
+              free(*str);
+              *str=gal_blank_as_string(GAL_TYPE_STRING, 0);
+            }
+          ++str;
+        }
+      while(++f<ff);
+      break;
+
+    /* Currently unsupported types. */
+    case GAL_TYPE_BIT:
+    case GAL_TYPE_COMPLEX32:
+    case GAL_TYPE_COMPLEX64:
+      error(EXIT_FAILURE, 0, "%s: %s type not yet supported",
+            __func__, gal_type_name(input->type, 1));
+
+    /* Bad input. */
+    default:
+      error(EXIT_FAILURE, 0, "%s: type value (%d) not recognized",
+            __func__, input->type);
+    }
+
+  /* Update the blank flags. */
+  gal_blank_present(input, 1);
+}
+
+
+
+
+
 /* Remove blank elements from a dataset, convert it to a 1D dataset and
    adjust the size properly. In practice this function doesn't `realloc'
    the input array, all it does is to shift the blank eleemnts to the end
@@ -363,93 +665,3 @@ gal_blank_remove(gal_data_t *input)
   input->flag |= GAL_DATA_FLAG_BLANK_CH;
   input->flag &= ~GAL_DATA_FLAG_HASBLANK;
 }
-
-
-
-
-
-/* Print the blank value as a string. For the integer types, we'll use the
-   PRIxNN keywords of `inttypes.h' (which is imported into Gnuastro from
-   Gnulib, so we don't necessarily rely on the host system having it). */
-char *
-gal_blank_as_string(uint8_t type, int width)
-{
-  char *blank=NULL, *fmt;
-
-  /* Print the given value. */
-  switch(type)
-    {
-    case GAL_TYPE_BIT:
-      error(EXIT_FAILURE, 0, "%s: bit types are not implemented yet",
-            __func__);
-      break;
-
-    case GAL_TYPE_STRING:
-      if(width) asprintf(&blank, "%*s", width,  GAL_BLANK_STRING);
-      else      asprintf(&blank, "%s",          GAL_BLANK_STRING);
-      break;
-
-    case GAL_TYPE_UINT8:
-      fmt = width ? "%*"PRIu8 : "%"PRIu8;
-      if(width) asprintf(&blank, fmt, width, (uint8_t)GAL_BLANK_UINT8);
-      else      asprintf(&blank, fmt,        (uint8_t)GAL_BLANK_UINT8);
-      break;
-
-    case GAL_TYPE_INT8:
-      fmt = width ? "%*"PRId8 : "%"PRId8;
-      if(width) asprintf(&blank, fmt, width, (int8_t)GAL_BLANK_INT8);
-      else      asprintf(&blank, fmt,        (int8_t)GAL_BLANK_INT8);
-      break;
-
-    case GAL_TYPE_UINT16:
-      fmt = width ? "%*"PRIu16 : "%"PRIu16;
-      if(width) asprintf(&blank, fmt, width, (uint16_t)GAL_BLANK_UINT16);
-      else      asprintf(&blank, fmt,        (uint16_t)GAL_BLANK_UINT16);
-      break;
-
-    case GAL_TYPE_INT16:
-      fmt = width ? "%*"PRId16 : "%"PRId16;
-      if(width) asprintf(&blank, fmt, width, (int16_t)GAL_BLANK_INT16);
-      else      asprintf(&blank, fmt,        (int16_t)GAL_BLANK_INT16);
-      break;
-
-    case GAL_TYPE_UINT32:
-      fmt = width ? "%*"PRIu32 : "%"PRIu32;
-      if(width) asprintf(&blank, fmt, width, (uint32_t)GAL_BLANK_UINT32);
-      else      asprintf(&blank, fmt,        (uint32_t)GAL_BLANK_UINT32);
-      break;
-
-    case GAL_TYPE_INT32:
-      fmt = width ? "%*"PRId32 : "%"PRId32;
-      if(width) asprintf(&blank, fmt, width, (int32_t)GAL_BLANK_INT32);
-      else      asprintf(&blank, fmt,        (int32_t)GAL_BLANK_INT32);
-      break;
-
-    case GAL_TYPE_UINT64:
-      fmt = width ? "%*"PRIu64 : "%"PRIu64;
-      if(width) asprintf(&blank, fmt, width, (uint64_t)GAL_BLANK_UINT64);
-      else      asprintf(&blank, fmt,        (uint64_t)GAL_BLANK_UINT64);
-      break;
-
-    case GAL_TYPE_INT64:
-      fmt = width ? "%*"PRId64 : "%"PRId64;
-      if(width) asprintf(&blank, fmt, width, (int64_t)GAL_BLANK_INT64);
-      else      asprintf(&blank, fmt,        (int64_t)GAL_BLANK_INT64);
-      break;
-
-    case GAL_TYPE_FLOAT32:
-      if(width) asprintf(&blank, "%*f", width,  (float)GAL_BLANK_FLOAT32);
-      else      asprintf(&blank, "%f",          (float)GAL_BLANK_FLOAT32);
-      break;
-
-    case GAL_TYPE_FLOAT64:
-      if(width) asprintf(&blank, "%*f", width,  (double)GAL_BLANK_FLOAT64);
-      else      asprintf(&blank, "%f",          (double)GAL_BLANK_FLOAT64);
-      break;
-
-    default:
-      error(EXIT_FAILURE, 0, "%s: type code %d not recognized", __func__,
-            type);
-    }
-  return blank;
-}
