@@ -4,7 +4,7 @@
 # (in the Installing gnuastro section).
 #
 # Original author:
-#     Mohammad Akhlaghi <akhlaghi@gnu.org>
+#     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 # Contributing author(s):
 #
 # Copying and distribution of this file, with or without modification,
@@ -53,4 +53,8 @@ if [ "x$haslibjpeg" != "xyes" ];then echo "libjpeg not present.";  exit 77;fi
 
 # Actual test script
 # ==================
-$execname blank $img blank --output=blankch.jpg
+#
+# `check_with_program' can be something like `Valgrind' or an empty
+# string. Such programs will execute the command if present and help in
+# debugging when the developer doesn't have access to the user's system.
+$check_with_program $execname blank $img blank --output=blankch.jpg
