@@ -52,4 +52,8 @@ if [ "x$haslibjpeg" != "xyes" ];then echo "libjpeg not present.";  exit 77;fi
 
 # Actual test script
 # ==================
-$execname blank blank blank $img --output=f2jcmyk.jpg
+#
+# `check_with_program' can be something like `Valgrind' or an empty
+# string. Such programs will execute the command if present and help in
+# debugging when the developer doesn't have access to the user's system.
+$check_with_program $execname blank blank blank $img --output=f2jcmyk.jpg
