@@ -51,4 +51,8 @@ if [ ! -f $img2     ]; then echo "$img2 does not exist.";  exit 77; fi
 
 # Actual test script
 # ==================
-$execname $img2 --copy="Mock profiles" --output=$img1
+#
+# `check_with_program' can be something like `Valgrind' or an empty
+# string. Such programs will execute the command if present and help in
+# debugging when the developer doesn't have access to the user's system.
+$check_with_program $execname $img2 --copy="Mock profiles" --output=$img1
