@@ -1,10 +1,10 @@
-# Detect objects and clumps in an image using NoiseChisel.
+# Detect objects in an image using NoiseChisel.
 #
 # See the Tests subsection of the manual for a complete explanation
 # (in the Installing gnuastro section).
 #
 # Original author:
-#     Mohammad Akhlaghi <akhlaghi@gnu.org>
+#     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 # Contributing author(s):
 #
 # Copying and distribution of this file, with or without modification,
@@ -41,7 +41,7 @@ img=convolve_spatial_noised.fits
 #   - The input data was not made (for example the test that created the
 #     data file failed).
 if [ ! -f $execname ]; then echo "$execname not created."; exit 77; fi
-if [ ! -f $img      ]; then echo "$img does not exist.";    exit 77; fi
+if [ ! -f $img      ]; then echo "$img does not exist.";   exit 77; fi
 
 
 
@@ -49,5 +49,10 @@ if [ ! -f $img      ]; then echo "$img does not exist.";    exit 77; fi
 
 # Actual test script
 # ==================
-$execname $img --cleandilated --checkdetection --checksegmentation \
-          --continueaftercheck
+#
+# `check_with_program' can be something like `Valgrind' or an empty
+# string. Such programs will execute the command if present and help in
+# debugging when the developer doesn't have access to the user's system.
+$check_with_program $execname $img --tilesize=100,100 --snquant=0.999 \
+                              --cleangrowndet --checkdetection        \
+                              --continueaftercheck
